// ==============================================================
// ORBITER vessel module: Juno II Pioneer
// ==============================================================

#include "orbitersdk.h"
#include "multi_stage.h"

const VECTOR3 OFS_TRANSPORTER = { 0,0,0};
const VECTOR3 OFS_MODULE      = { 0.0, OFS_TRANSPORTER.y, OFS_TRANSPORTER.z+2.5};
const VECTOR3 OFS_POD         = { 0,OFS_TRANSPORTER.y+.5,OFS_TRANSPORTER.z+16.0};



//const VECTOR3 OFS_POD         = { 0,0,0};
//const VECTOR3 OFS_MODULE      = { 0.0, -.5, -13.5};
//const VECTOR3 OFS_TRANSPORTER = { 0,OFS_MODULE.y,OFS_MODULE.z-2.5};
bool bPodSeparate = false;
bool bModuleSeparate = false;
double stage_sep=0.0;
UINT stage = 0;

void AddTransporterExhaust(VESSEL *vessel)
{
	UINT atthand;
	VECTOR3 att_ref;
	VECTOR3 att_dir;
	VECTOR3 m_exhaust_pos;
	VECTOR3 m_exhaust_ref;
	vessel->ClearAttExhaustRefs();
	vessel->ClearExhaustRefs();
	//main
	m_exhaust_ref = _V(0,0,-1);
	m_exhaust_pos= _V(-0.001084, -0.754090, -13.186956);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 30.0, 1.0, &m_exhaust_ref);
	m_exhaust_pos= _V(2.447956, 0.586273, -13.186956);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 30.0, 1.0, &m_exhaust_ref);
	m_exhaust_pos= _V(-2.407538, 0.577893, -13.186956);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 30.0, 1.0, &m_exhaust_ref);
	m_exhaust_pos= _V(-0.001084, 1.911590, -13.186956);
	vessel->AddExhaustRef (EXHAUST_MAIN, m_exhaust_pos, 30.0, 1.0, &m_exhaust_ref);
	//retro


	m_exhaust_ref = _V(0,0,1);
	m_exhaust_pos= _V(6.224432, -.3, 10.738448);
	vessel->AddExhaustRef (EXHAUST_RETRO, m_exhaust_pos, 1.30, .10, &m_exhaust_ref);
	m_exhaust_pos= _V(6.224432, -.3, -5.073767);
	vessel->AddExhaustRef (EXHAUST_RETRO, m_exhaust_pos, 1.30, .10, &m_exhaust_ref);
	m_exhaust_pos= _V(-6.224432, -.3, 10.738448);
	vessel->AddExhaustRef (EXHAUST_RETRO, m_exhaust_pos, 1.30, .10, &m_exhaust_ref);
	m_exhaust_pos= _V(-6.224432, -.3, -5.073767);
	vessel->AddExhaustRef (EXHAUST_RETRO, m_exhaust_pos, 1.30, .10, &m_exhaust_ref);
	//hover
	m_exhaust_ref = _V(0,-1,0);
	m_exhaust_pos= _V(0.633429, -1.553534, 12.017182);
	vessel->AddExhaustRef (EXHAUST_HOVER, m_exhaust_pos, 5.0, 0.5, &m_exhaust_ref);
	m_exhaust_pos= _V(-0.634928, -1.553534, 12.021487);
	vessel->AddExhaustRef (EXHAUST_HOVER, m_exhaust_pos, 5.0, 0.5, &m_exhaust_ref);
	m_exhaust_pos= _V(-0.634928, -1.553088, -6.698983);
	vessel->AddExhaustRef (EXHAUST_HOVER, m_exhaust_pos, 5.0, 0.5, &m_exhaust_ref);
	m_exhaust_pos= _V(0.641072, -1.553088, -6.703419);
	vessel->AddExhaustRef (EXHAUST_HOVER, m_exhaust_pos, 5.0, 0.5, &m_exhaust_ref);
	if (stage ==0)
	{
		m_exhaust_pos= _V(1.003072, -1.877490, 6.585338);
		vessel->AddExhaustRef (EXHAUST_HOVER, m_exhaust_pos, 5.0, 0.5, &m_exhaust_ref);
		m_exhaust_pos= _V(-0.996928, -1.877490, 6.585338);
		vessel->AddExhaustRef (EXHAUST_HOVER, m_exhaust_pos, 5.0, 0.5, &m_exhaust_ref);
		m_exhaust_pos= _V(-0.996928, -1.877490, -1.304029);
		vessel->AddExhaustRef (EXHAUST_HOVER, m_exhaust_pos, 5.0, 0.5, &m_exhaust_ref);
		m_exhaust_pos= _V(1.003072, -1.877490, -1.304029);
		vessel->AddExhaustRef (EXHAUST_HOVER, m_exhaust_pos, 5.0, 0.5, &m_exhaust_ref);
	}
	//up

	att_dir = _V(0,-1,0);
  	att_ref = _V(6.224432, -.3, 10.238448);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	att_ref = _V(-6.224432, -.3, 10.238448);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	att_dir = _V(0,1,0);
	att_ref = _V(6.224432, -.3, -5.573767);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	att_ref = _V(-6.224432, -.3, -5.573767);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	//dn
	
	att_dir = _V(0,1,0);
  	att_ref = _V(6.224432, -.3, 10.238448);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);

	att_ref = _V(-6.224432, -.3, 10.238448);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);

	att_dir = _V(0,-1,0);
	att_ref = _V(6.224432, -.3, -5.573767);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);

	att_ref = _V(-6.224432, -.3, -5.573767);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);
	
	
	//right

	
	att_dir = _V(0,0,1);

	att_ref = _V(6.224432, -.3, 10.238448);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.6);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);
	

	att_dir = _V(0,0,-1);

	att_ref = _V(-6.224432, -.3, -5.573767);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.6);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);
	

	//left
	att_dir = _V(0,0,1);

	att_ref = _V(-6.224432, -.3, 10.238448);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.6);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);
	

	att_dir = _V(0,0,-1);

	att_ref = _V(6.224432, -.3, -5.573767);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.6);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);
	
	//rot

	att_dir = _V(0,1,0);

	att_ref = _V(6.224432, -.3, 10.238448);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,0);

	att_ref = _V(6.224432, -.3, -5.573767);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,0);

	att_dir = _V(0,-1,0);

	att_ref = _V(-6.224432, -.3, 10.238448);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,0);

	att_ref = _V(-6.224432, -.3, -5.573767);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,0);

	att_dir = _V(0,-1,0);

	att_ref = _V(6.224432, -.3, 10.238448);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,1);

	att_ref = _V(6.224432, -.3, -5.573767);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,1);

	att_dir = _V(0,1,0);

	att_ref = _V(-6.224432, -.3, 10.238448);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,1);

	att_ref = _V(-6.224432, -.3, -5.573767);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,1);

	//tran up
	att_dir = _V(0,-1,0);
	
	att_ref = _V(6.224432, -.3, 10.238448);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,0.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,1,0);
	
	att_ref = _V(6.224432, -.3, -5.573767);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,0.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,1,0);

	att_ref = _V(-6.224432, -.3, 10.238448);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,0.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,1,0);

	att_ref = _V(-6.224432, -.3, -5.573767);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,0.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,1,0);
	//tran down
	att_dir = _V(0,1,0);
	att_ref = _V(6.224432, -.3, 10.238448);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,0.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,1,1);

	att_ref = _V(6.224432, -.3, -5.573767);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,0.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,1,1);
	
	att_ref = _V(-6.224432, -.3, 10.238448);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,0.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,1,1);
	att_ref = _V(-6.224432, -.3, -5.573767);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,0.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,1,1);
	//tran left
	att_dir = _V(1,0,0);
	att_ref = _V(6.224432, -.3, 10.238448);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,0,1);
	att_ref = _V(6.224432, -.3, -5.573767);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,0,1);
	//tran right
	att_dir = _V(-1,0,0);
	att_ref = _V(-6.224432, -.3, 10.238448);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,0,0);
	att_ref = _V(-6.224432, -.3, -5.573767);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.1,.3);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,0,0);
}

void AddPodExhaust(VESSEL *vessel)
{
	UINT atthand;
	VECTOR3 att_ref;
	VECTOR3 att_dir;
	vessel->ClearAttExhaustRefs();
	vessel->ClearExhaustRefs();
	//up
	att_dir = _V(0,0,-1);

	att_ref = _V(0.060736, -1.840258, -1.223674);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	att_dir = _V(0,0,1);
	att_ref = _V(0.012348, 1.813132, -0.867834);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,0);

	//dn
	
	att_dir = _V(0,0,1);
	att_ref= _V(0.036099, -1.813132, -0.867834);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);
	
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);
	
	att_dir = _V(0,0,-1);
	att_ref= _V(-0.012289, 1.840259, -1.223673);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);
	
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,0,1);
	
	
	//right
	att_ref= _V(-2.922896, 0.021065, -1.128998);
	att_dir = _V(0,0,-1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);
	
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);
	
	att_dir = _V(0,0,1);

	att_ref= _V(2.869782, 0.094512, -0.830441);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);
	
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,1);

	//left
	att_dir = _V(0,0,-1);

	att_ref= _V(2.922896, 0.021065, -1.128998);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);
	
	att_ref= _V(2.922896, 0.021065, -1.128998);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);

	att_dir = _V(0,0,1);

	att_ref= _V(-2.863716, 0.007508, -0.830441);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);
	
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,1,0);
	
	//rot
	att_dir = _V(-1,0,0);
	att_ref = _V(-0.279569, 1.840608, -1.050051);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,0);

	att_dir = _V(1,0,0);
	att_ref = _V(0.328016, -1.840608, -1.050051);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,0);

	att_dir = _V(-1,0,0);
	att_ref = _V(-0.209617, -1.839300, -1.036478);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,1);

	att_dir = _V(1,0,0);
	att_ref = _V(0.258063, 1.839299, -1.036478);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_ROT,2,1);
/*
	//tran up
	att_dir = _V(0,-1,0);
	att_ref= _V(0.060736, -1.840258, -1.302298);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,1,0);

	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,1,0);
	//tran down
	att_ref= _V(0.036099, -1.813132, -0.946459);
	att_dir = _V(0,1,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,1,1);

	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,1,1);
	//tran left
	att_ref = _V(-0.012289, 1.840259, -1.153524);
	att_dir = _V(1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,0,1);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,0,1);
	//tran right
	att_ref = _V(0.012348, 1.813132, -0.797684);
	att_dir = _V(-1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,0,0);
	att_ref = _V(-0.15,0.55,-.25);
	att_dir = _V(-1,0,0);
	atthand = vessel->AddAttExhaustRef(att_ref,att_dir,0.02,0.1);
	vessel->AddAttExhaustMode(atthand,ATTMODE_LIN,0,0);
*/
}

void SetPod (VESSEL *vessel)
{
// ==============================================================
	vessel->SetSize (17.0);
	vessel->SetCOG_elev (3.29);
	vessel->SetEmptyMass (800);
	vessel->SetMaxFuelMass (5);
	vessel->SetFuelMass (5);
	vessel->SetISP (5000);
	vessel->SetCameraOffset(_V(-1.304799, 0.757537, -1.392108));
	vessel->SetMaxThrust (ENGINE_MAIN, 250);
	vessel->SetMaxThrust (ENGINE_RETRO, 8.2);
	vessel->SetMaxThrust (ENGINE_HOVER, 0);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 8.2);
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(2.94, 3.74, 2.09));
	vessel->SetCrossSections (_V(15.80, 24.12, 16.08));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.2, 0.2, 1.5,1.5);
	vessel->SetRotDrag (_V(0.5,0.5,0.5));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->SetDockParams (_V(-0.016000, -0.091037, -2.987434), _V(0,0,-1), _V(0,1,0));
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	vessel->AddMesh ("eagle1p");
	vessel->ShiftCentreOfMass(OFS_POD);
	AddPodExhaust(vessel);
}

void SetNoModule (VESSEL *vessel)
{
// ==============================================================
	vessel->SetSize (17.0);
	vessel->SetCOG_elev (3.29);
	vessel->SetEmptyMass (7575);
	vessel->SetMaxFuelMass (16000);
	if (vessel->GetFuelMass() > 16000)
		vessel->SetFuelMass(16000);
	vessel->SetISP (50000);
	vessel->SetCameraOffset(_V(-1.327018, 1.422520, 11.723770));
	vessel->SetMaxThrust (ENGINE_MAIN, 990000);
	vessel->SetMaxThrust (ENGINE_RETRO, 66000);
	vessel->SetMaxThrust (ENGINE_HOVER, 495000 / 2);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 66000);
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(85.02, 92.82, 12.55));
	vessel->SetCrossSections (_V(127.79, 275.81, 90.03));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.2, 0.2, 1.5,1.5);
	vessel->SetRotDrag (_V(0.7,0.7,0.5));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->SetDockParams (_V(3.095514, 0.138607, -0.234480), _V(1,0,0), _V(0,1,0));
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	vessel->AddMesh ("eagle1y", &OFS_TRANSPORTER);
	vessel->AddMesh ("eagle1p", &OFS_POD);
	AddTransporterExhaust(vessel);
}

void SetTransporter (VESSEL *vessel)
{
// ==============================================================
	vessel->SetSize (17.0);
	vessel->SetCOG_elev (3.29);
	vessel->SetEmptyMass (9000);
	vessel->SetMaxFuelMass (24000);
	vessel->SetISP (50000);
	vessel->SetCameraOffset(_V(-1.327018, 1.422520, 11.723770));
	vessel->SetMaxThrust (ENGINE_MAIN, 990000);
	vessel->SetMaxThrust (ENGINE_RETRO, 66000);
	vessel->SetMaxThrust (ENGINE_HOVER, 495000);
	vessel->SetMaxThrust (ENGINE_ATTITUDE, 66000);
// ==============================================================
// This from running the mesh through the shipedit program
	vessel->SetPMI (_V(127.85, 132.79, 12.53));
	vessel->SetCrossSections (_V(188.92, 384.65, 91.36));
// ==============================================================
// This is based off of the SRB and Main Shuttle Tank as 
// they are the nearest thing to V-2 that I have stats for.
	vessel->SetCW (0.2, 0.2, 1.5,1.5);
	vessel->SetRotDrag (_V(0.7,0.7,0.5));
	vessel->SetPitchMomentScale (0);
	vessel->SetBankMomentScale (0);
	vessel->SetLiftCoeffFunc (0); 
	vessel->SetDockParams (_V(3.095514, 0.138607, 2.5), _V(1,0,0), _V(0,1,0));
	vessel->ClearMeshes();
	vessel->ClearExhaustRefs();
	vessel->ClearAttExhaustRefs();
	vessel->AddMesh ("eagle1y", &OFS_TRANSPORTER);
	vessel->AddMesh ("eagle1p", &OFS_POD);
	vessel->AddMesh ("eagle1m", &OFS_MODULE);
	AddTransporterExhaust(vessel);
}

void SeparatePod (VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,0);
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	ofs = OFS_TRANSPORTER;	// The same as whatever offset you used to place the stage 
	vel = _V(0,0,-2);	    // Staging Velocity represents 
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;	//<- Use these to rotate the ejected stage
	vs.vrot.y = 0.0;	//<-
	vs.vrot.z = 0.0;	//<-
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Transporter");
	VESSEL::Create (name, "Eagle99Trans", vs);
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
}

void SeparateNoModule (VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,0);
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	ofs = OFS_TRANSPORTER;	// The same as whatever offset you used to place the stage 
	vel = _V(0,0,-2);	    // Staging Velocity represents 
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;	//<- Use these to rotate the ejected stage
	vs.vrot.y = 0.0;	//<-
	vs.vrot.z = 0.0;	//<-
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Transporter");
	VESSEL::Create (name, "Eagle99TransNoMod", vs);
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
}

void SeparateModule (VESSEL *vessel)
{
	VESSELSTATUS vs;
	VECTOR3 ofs = _V(0,0,0);
	VECTOR3 vel = _V(0,0,0);
	vessel->SetEngineLevel(ENGINE_MAIN, 0);
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	ofs = OFS_MODULE;	// The same as whatever offset you used to place the stage 
	vel = _V(0,-3,0);	    // Staging Velocity represents 
	StageTransform(vessel,&vs,ofs,vel);
	vs.vrot.x = 0.0;	//<- Use these to rotate the ejected stage
	vs.vrot.y = 0.0;	//<-
	vs.vrot.z = 0.0;	//<-
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-Module");
	VESSEL::Create (name, "Eagle99Module", vs);
}

void SeparateEVA (VESSEL *vessel)
{
	VESSELSTATUS vs;
	vessel->GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vs.fuel=1.0;
	VECTOR3 ofs = _V(3.095514, 0.138607, -0.234480);
	VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
	VECTOR3 vel = {0,0,0};
	vessel->Local2Rel (ofs, vs.rpos);
	vessel->GlobalRot (vel, rofs);
	char name[256];
	strcpy (name, vessel->GetName()); strcat (name, "-EVA");
	VESSEL::Create (name, "astro1999", vs);
}


// ==============================================================
// API interface
// ==============================================================



// Initialisation
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	stage = 0;
	return new VESSEL (hvessel, flightmodel);
}

// Cleanup
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete vessel;
}

// Keyboard interface handler
DLLCLBK int ovcConsumeKey (VESSEL *vessel, const char *keystate)
{
	OBJHANDLE hvessel;
	if (KEYMOD_SHIFT (keystate)) 
	{
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else 
	{ // unmodified keys
		if (KEYDOWN (keystate, OAPI_KEY_J)) { // "Jettison Pod"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
				bPodSeparate = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_M)) { // "Eject Module"
			if (oapiAcceptDelayedKey (OAPI_KEY_M, 1.0))
				bModuleSeparate = true;
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_E)) { // "EVA"
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
				{
					char name[256];
					strcpy (name, vessel->GetName()); strcat (name, "-EVA");
					hvessel=oapiGetVesselByName(name);
					if (hvessel == 0)
					{
						SeparateEVA(vessel);
						hvessel=oapiGetVesselByName(name);
						if (hvessel != 0)
							oapiSetFocusObject(hvessel);
					}
					else
					{
						hvessel=oapiGetVesselByName(name);
						if (hvessel != 0)
							oapiSetFocusObject(hvessel);
					};
				};
			return 1;
		}		
	}
	return 0;
}

// Set the capabilities of the vessel class
DLLCLBK void ovcSetClassCaps (VESSEL *vessel, FILEHANDLE cfg)
{
	SetTransporter(vessel);
}


DLLCLBK void ovcTimestep (VESSEL *vessel, double simt)
{
	if (stage < 2)
	{
		if (bPodSeparate)
		{
			if (stage == 0)
				SeparateNoModule(vessel);
			else
				SeparatePod(vessel);
			SetPod(vessel);
			stage = 2;
		} 
		if (bModuleSeparate && stage == 0)
		{
			SeparateModule(vessel);
			stage = 1;
			SetNoModule(vessel);
		} 
		bPodSeparate=false;
		bModuleSeparate=false;
	}

}

DLLCLBK void ovcLoadState (VESSEL *vessel, FILEHANDLE scn, VESSELSTATUS *def_vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "STAGE", 5)) {
			sscanf (line+5, "%d", &stage);
			switch (stage) {
			case 0:
				SetTransporter(vessel);
				break;
			case 1:
				SetNoModule(vessel);
				break;
			case 2:
				SetPod(vessel);
				break;
			}
		} else {
			vessel->ParseScenarioLine (line, def_vs);
		}
    }
}
//

// Save status to scenario file
DLLCLBK void ovcSaveState (VESSEL *vessel, FILEHANDLE scn)
{
	vessel->SaveDefaultState(scn);
	oapiWriteScenario_int (scn, "STAGE", stage);
}

DLLCLBK void ovcSetState (VESSEL *vessel, const VESSELSTATUS *status)
{
	vessel->DefSetState (status);	        // default initialisation
}